#!/bin/bash
#
# 2013/03/07
# Federico Bareilles <bareilles@gmail.com>
# 2013/09/12
# Jorge Secreto <jorgesecreto@gmail.com>
#    SOLDESA es un desarrollo del Departamento de Evaluacion Sensorial de
#    Alimentos del Instituto Superior Experimental de Tecnologia Alimentaria
#    (DESA-ISETA) y de la Comision de Investigaciones Cientificas de la provincia
#    de Buenos Aires (CIC), con la colaboracion de la Cooperativa de Software 
#    Libre ELVEX
#    Mail: graciela@desa.edu.ar
#    Sitio Web: www.desa.edu.ar    
#
#    Copyright (c) 2013 DESA/ISETA   
#    This file is part of Soldesa.
#
#    Soldesa is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Soldesa is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Soldesa.  If not, see <http://www.gnu.org/licenses/>.

APTGETOPT="-qq -y"
SOLDESAUSER=soldesa
WD=$(pwd)
echo=$(which echo)

usage() {
    echo ""
    echo "Use: $0 [-h] [-u username] [-d]"
    echo "  -u: Set user name for soldesa, default = ${SOLDESAUSER}."
    echo "  -d: Uninstall soldesa" 
    echo "  -h: This help."
    echo ""
}

if_deb_inst() {
    ret=1
    pkg=$1
    result=$(dpkg -s $pkg|grep ^Status)
    if echo $result|grep -Eq " ok "; then
	if [ "$(echo $result|sed 's/.*ok //')" = "installed" ]; then
	    ret=0
	fi
    fi
    return $ret
}

get_home() {
    grep -E "^${1}:" /etc/passwd|awk -F ":" '{print $6}'
}

get_gid() {
    grep -E "^${1}:" /etc/passwd|awk -F ":" '{print $4}'
}

uninstall() {
    if  ps -A|grep -q gunicorn ; then 
       killall gunicorn
    fi
    if [ -f /etc/init.d/soldesa.sh ]; then
      TARGETDIR=$(grep SRC= /etc/init.d/soldesa.sh |awk -F '"' '{ print( $2) }' |awk -F '/src' '{ print($1) }')
      if [ -d $TARGETDIR ]; then
        SOLDESAUSER=$(grep $TARGETDIR /etc/passwd |awk -F ':' '{ print $1 }')
        if [ ! -z $SOLDESAUSER ]; then
          echo "Deleting user $SOLDESAUSER"
          deluser --remove-home --quiet $SOLDESAUSER
        else
          echo "Deleting $TARGETDIR"
          rm -r -f $TARGETDIR
        fi
      fi
      echo "Deleting /etc/init.d/soldesa.sh"
      rm /etc/init.d/soldesa.sh
      /usr/sbin/update-rc.d soldesa.sh remove
    fi

}
#------------------------------- main --------------------------------------

while getopts 'du:' opt
do
    case $opt in
    u) SOLDESAUSER=$OPTARG ;;
    d) uninstall
       exit 0
       ;;
    esac
done
shift $(($OPTIND - 1))


if [ ! -d ${WD}/src ]; then
    $echo -e "\nSource directory src not found in this directory:"
    $echo -e "$WD\n"
    exit 1;
fi

if ! grep -qE "^${SOLDESAUSER}:" /etc/passwd; then
    $echo "Creating soldesa user: ${SOLDESAUSER}"
    adduser --system --shell /bin/false --disabled-login ${SOLDESAUSER}
fi

TARGETDIR=$(get_home ${SOLDESAUSER})

if [ ! -d $TARGETDIR ]; then
    echo "Creating soldesa target directory: $TARGETDIR"
    mkdir -p $TARGETDIR
fi

cp -a  $WD/src $TARGETDIR/
chown -R ${SOLDESAUSER}:$(get_gid ${SOLDESAUSER}) $TARGETDIR/src

#----------- debian/ubuntu espesific: --------------------------------------

if [ -f /etc/debian_version ]; then
    DEBPKG=""
    if ! if_deb_inst python-flask ; then 
	DEBPKG="$DEBPKG python-flask"
    fi
    if ! if_deb_inst python-paste ; then 
	DEBPKG="$DEBPKG python-paste"
    fi
    if ! if_deb_inst couchdb ; then 
	DEBPKG="$DEBPKG couchdb"
    fi
    if ! if_deb_inst python-virtualenv ; then 
	DEBPKG="$DEBPKG python-virtualenv"
    fi
    if [ -n "$DEBPKG" ]; then
	apt-get update
	apt-get $APTGETOPT install $DEBPKG
    fi


fi
#--- end: debian/ubuntu -----------------------------------------------------

cd ${TARGETDIR}/src/soldesa/
virtualenv env
. env/bin/activate
pip install $(cat deps)
python manage.py insert_users 2>/dev/null
cd $WD

cat<<EOF > /etc/init.d/soldesa.sh
#! /bin/sh
### BEGIN INIT INFO
# Provides:          soldesa
# Required-Start:    couchdb
# Required-Stop:
# Default-Start:     2 3 4 5
# Default-Stop:
# Short-Description: soldesa initialization.
# Description:       soldesa initialization.
#                    
### END INIT INFO

export SRC="${TARGETDIR}/src/soldesa"
export PYTHONPATH=\$SRC:\$PYTHONPATH

case "\$1" in
  start|"")
        \$SRC/env/bin/gunicorn -w 6 --log-level debug --log-file \$SRC/soldesa.log wsgi:app -b 0.0.0.0 -D
        ;;
  restart|reload|force-reload)
        echo "Error: argument \$1 not supported" >&2
        exit 3
        ;;
  stop)
        # No-op
        echo "Error: argument \$1 not supported" >&2
        exit 3
        ;;
  *)
        echo "Usage: soldesa.sh [start]" >&2
        exit 3
        ;;
esac

:
EOF

chmod +x /etc/init.d/soldesa.sh
/usr/sbin/update-rc.d soldesa.sh start
/etc/init.d/soldesa.sh start
