#    SOLDESA es un desarrollo del Departamento de Evaluacion Sensorial de
#    Alimentos del Instituto Superior Experimental de Tecnologia Alimentaria
#    (DESA-ISETA) y de la Comision de Investigaciones Cientificas de la provincia
#    de Buenos Aires (CIC), con la colaboracion de la Cooperativa de Software 
#    Libre ELVEX
#    Mail: graciela@desa.edu.ar
#    Sitio Web: www.desa.edu.ar    
#
#    Copyright (c) 2013 DESA/ISETA   
#    This file is part of Soldesa.
#
#    Soldesa is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Soldesa is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Soldesa.  If not, see <http://www.gnu.org/licenses/>.
import socket

from flask import Flask, g
from flaskext.couchdb import CouchDBManager, Document
import model
from flaskext.principal import Principal, identity_loaded, RoleNeed

app = Flask(__name__)
couch_manager = CouchDBManager(auto_sync=False)
import views

def setup_app(config=None):
    app.config.from_object('config.Config')

    # Setup Document in CouchDB
    couch_manager.setup(app)

    for name in dir(model):
        item = getattr(model, name)
        if isinstance(item, type) and issubclass(item, Document):
            couch_manager.add_document(item)
    try:
        couch_manager.sync(app)
    except socket.error, error:
        print error
        print 'Inicializaste couchdb?'
        exit()

    p = Principal(app)
    import perms
    return app

def _make_context():
    # CouchDB h4x for shell
    g.couch = couch_manager.connect_db(app)
    return dict(app=setup_app(), model=model)

def export_xls():
    import xlwt

    g.couch = couch_manager.connect_db(app)
    # Create workbook and worksheet
    wbk = xlwt.Workbook()
    sheet = wbk.add_sheet('Tuitos')


    header = ['Panel_Idpanel', 'Nombre', 'Fecha', 'Hora', 'REP', 'Tope', 
            'EVALUA', 'IdEvaluador', 'MUESTRA', 'Producto', 'VASITO', 'ORDEN']

    panel = model.Panel.load('971a9815624a77578fc8409e28099df9')
    panel_id = panel.id
    panel_nombre = panel.nombre

    panel_start = [panel_id]
    panel_end = [panel_id, []]

    descriptores = [model.Descriptor.load(i) for i in panel.descriptores]

    for desc in descriptores:
        for attr in desc.atributos:
            header.append(desc.nombre.upper()+attr)

    for i, h in enumerate(header):
        sheet.write(0, i, h)

    users = {}
    row = 0
    for i in model.Session.view_panel_ids(reduce=False)[panel_start:panel_end]:
        row += 1

        user = users.get(i.id_evaluador, model.User.load(i.id_evaluador))

        sheet.write(row, header.index('Panel_Idpanel'), panel_id)
        sheet.write(row, header.index('Nombre'), panel_nombre)
        #TODO: Fecha, Hora
        sheet.write(row, header.index('REP'), i.repeticion)
        sheet.write(row, header.index('Tope'), i.tope)
        sheet.write(row, header.index('EVALUA'), i.evaluador)
        sheet.write(row, header.index('IdEvaluador'), user.username)
        sheet.write(row, header.index('MUESTRA'), i.muestra)
        sheet.write(row, header.index('Producto'), i.producto)
        sheet.write(row, header.index('VASITO'), i.vasito)
        sheet.write(row, header.index('ORDEN'), i.orden)

        for descriptor in descriptores:
            for attr in descriptor.atributos:
                key = descriptor.nombre.upper()+attr
                if descriptor.id in i.descriptores and \
                        attr in i.descriptores[descriptor.id]:
                    value = int(i.descriptores[descriptor.id][attr])
                else:
                    value = 0
                sheet.write(row, header.index(key), value)
    wbk.save('sadasa.xls')




def insert_users():
    ''' Add default users to couchdb '''
    # CouchDB h4x for shell
    g.couch = couch_manager.connect_db(app)

    user = model.User(username="admin", roles=["admin"])
    user.set_password("123456")
    user.store()

    user = model.User(username="lider", password="123456",
                                          roles=["lider"])
    user.store()

    for i in range(4):
        user = model.User(username="user%s" % i, roles=["user"])
        user.set_password("123456")
        user.store()

    user = model.User(username="director", roles=["director"])
    user.set_password("123456")
    user.store()

    print 'Se crearon los usuarios "user", "lider", "admin", "director"'
