#    SOLDESA es un desarrollo del Departamento de Evaluacion Sensorial de
#    Alimentos del Instituto Superior Experimental de Tecnologia Alimentaria
#    (DESA-ISETA) y de la Comision de Investigaciones Cientificas de la provincia
#    de Buenos Aires (CIC), con la colaboracion de la Cooperativa de Software 
#    Libre ELVEX
#    Mail: graciela@desa.edu.ar
#    Sitio Web: www.desa.edu.ar    
#
#    Copyright (c) 2013 DESA/ISETA   
#    This file is part of Soldesa.
#
#    Soldesa is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Soldesa is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Soldesa.  If not, see <http://www.gnu.org/licenses/>.
from flaskext.wtf import Form, TextField, PasswordField, FileField,\
    SelectField, FieldList, IntegerField, FloatField, BooleanField,\
    SelectMultipleField, widgets, validators

class LoginForm(Form):
    username = TextField('Nombre', [validators.required()])
    password = PasswordField('Clave', [validators.required()])

class PanelForm(Form):
    nombre = TextField('Nombre', [validators.required()])
    lider = SelectField('Lider', [validators.required()])
    orden = FileField('Orden')

class MultiCheckboxField(SelectMultipleField):
    """
    A multiple-select, except displays a list of checkboxes.

    Iterating the field will produce subfields, allowing custom rendering of
    the enclosed checkbox fields.
    """
    widget = widgets.ListWidget(prefix_label=False)
    option_widget = widgets.CheckboxInput()

class ContributorForm(Form):
    lider = MultiCheckboxField(' ', [])

class EscalaForm(Form):
    min_escala = FloatField('Pos. Inicial',
            [], default=0)
    max_escala = FloatField('Pos. Final',
            [validators.required()], default=10)
    ancla = FloatField('Medio', [validators.required()], default=5)

class CreateUserForm(Form):
    username = TextField('Nombre', [validators.required()])
    password = PasswordField('Clave')
    role = SelectField('Role', [validators.required()], choices=[
        ('user', 'evaluador'), ('lider', 'lider')])
    group = SelectField('Grupo', [validators.required()])
    new_group = TextField('Crear Grupo')


class AtributoForm(Form):
    nombre = TextField('Nombre', [validators.required()])
    descriptores = FieldList(BooleanField(None), min_entries=1)


class GIForm(Form):
    ''' Form class with gettitem support '''
    def __getitem__(self, name):
        return getattr(self, name)

def dynform(fields):
    ''' return a form class with dynamic fields '''
    DynForm = type('DynForm', (GIForm, ), {})
    for field in fields:
        # TODO: better handling
        if isinstance(field, (tuple, list)):
            _type, name = field[0].split(':', 1)
            selfield = SelectField(None,
                    [validators.required()], choices=field[1])
            setattr(DynForm, name, selfield)
        else:
            _type, name = field.split(':', 1)
            if _type == 'int':
                setattr(DynForm, name, IntegerField('',
                                [validators.required()]))
            elif _type == 'chr':
                setattr(DynForm, name, TextField(None, [validators.required()]))
            else:
                raise Exception, 'Unknown type "%s"' % _type

    return DynForm
