#    SOLDESA es un desarrollo del Departamento de Evaluacion Sensorial de
#    Alimentos del Instituto Superior Experimental de Tecnologia Alimentaria
#    (DESA-ISETA) y de la Comision de Investigaciones Cientificas de la provincia
#    de Buenos Aires (CIC), con la colaboracion de la Cooperativa de Software 
#    Libre ELVEX
#    Mail: graciela@desa.edu.ar
#    Sitio Web: www.desa.edu.ar    
#
#    Copyright (c) 2013 DESA/ISETA   
#    This file is part of Soldesa.
#
#    Soldesa is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Soldesa is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Soldesa.  If not, see <http://www.gnu.org/licenses/>.
import os
import locale
import datetime

from tempfile import mkstemp
try:
    import simplejson as json
except ImportError:
    import json

# Flask Imports
from flaskext.principal import Identity, identity_changed
from flask import request, redirect, session, g, flash, send_file
from werkzeug import secure_filename

# My Application Imports
from couchie import app
from couchie.model import User, Panel, Session, Atributo
from couchie.perms import admin_permission, user_permission,\
                            lider_permission, logged
from couchie.utils import templated
from couchie.forms import LoginForm, PanelForm, CreateUserForm,\
                   AtributoForm, EscalaForm, ContributorForm, dynform

# External Imports
import xlrd
import xlwt
from xlrd import XLRDError


ALLOWED_EXTENSIONS = set(['xls'])

locale.setlocale(locale.LC_ALL, "es_AR.UTF-8")

def allowed_file(filename):
    ''' Check for a valid extension in filename '''
    if '.' in filename:
        if filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS:
            return True
    return False


def affect_atributo(panel_id, atributo_id, add=True):
    panel = Panel.load(panel_id)
    if panel:
        if add:
            if atributo_id not in panel.atributos:
                panel.atributos.append(atributo_id)
        else:
            if atributo_id in panel.atributos:
                panel.atributos.remove(atributo_id)
        panel.store()
    return ''


def get_descriptores(atributo_id):
    selected = []
    descriptores = []

    if atributo_id:
        if isinstance(atributo_id, Atributo):
            atributo = atributo_id
        else:
            atributo = Atributo.load(atributo_id)

        selected = sorted_descs(atributo)

        start = [atributo.nombre.lower()]
        end = [atributo.nombre.lower(), {}]

        descriptores = Atributo.view_descriptores[start:end]
        descriptores = [desc['key'][1] for desc in \
                sorted(descriptores, key=lambda i:i['key'][1], 
                                         cmp=locale.strcoll) \
                        if desc['key'][1] not in selected]


    return (descriptores, selected)

@app.route("/paneles/sort_attrs/<panel_id>", methods=("GET", "POST"))
@logged([admin_permission, lider_permission])
def sort_atributos(panel_id):
    attrs = request.form.getlist('tab[]')
    panel = Panel.load(panel_id)
    panel.atributos = attrs
    panel.store()
    return json.dumps({"sorted": True})

@app.route("/atributo/delete/<panel_id>/<atributo_id>")
@logged([admin_permission, lider_permission])
def delete_atributo(panel_id, atributo_id):
    affect_atributo(panel_id, atributo_id, add=False)
    return json.dumps({"removed": True})

@app.route("/atributo/create/<panel_id>", methods=("GET","POST"))
@app.route("/atributo/edit/<panel_id>/<atributo_id>", methods=("GET","POST"))
@logged([admin_permission, lider_permission])
@templated("create_atributo.html")
def create_atributo(panel_id, atributo_id=None):
    ''' Creates a Atributo '''
    form = AtributoForm()

    if atributo_id:
        atributo = Atributo.load(atributo_id)
        # workarround to use old form atributes
        if not atributo.anclas:
            atributo.anclas = {}
        all_desc, selected_desc = get_descriptores(atributo)
    else:
        atributo = Atributo()
        all_desc, selected_desc = get_descriptores(atributo_id)

    if form.validate_on_submit():
        form_descs = request.form.getlist('descriptores')

        atributo.nombre = form.nombre.data

        desc_atributos = {}
        for index, desc in enumerate(form_descs):
            desc = desc.encode("utf8")
            desc_atributos[desc] = index

            key = "ancla_left_"+desc
            atributo.anclas[key.decode("utf8")] = request.form.get(key)

            key = "ancla_rigth_"+desc
            atributo.anclas[key.decode("utf8")] = request.form.get(key)

        atributo.descriptores = desc_atributos
        atributo.store()

        status = {'status': 'ok', 'nombre': form.nombre.data,
                'id': atributo.id, 'atributos': desc_atributos.keys(),
                'descriptores': all_desc}

        affect_atributo(panel_id, atributo.id)
        return json.dumps(status)

    anclas = {}
    for desc in selected_desc:
        key = "ancla_left_"+desc
        anclas[key] = atributo.anclas.get(key, "Poco")
        key = "ancla_rigth_"+desc
        anclas[key] = atributo.anclas.get(key, "Mucho")


    name = atributo.nombre
    if name:
        form.nombre.data = name

    return {
        'form': form, 'descriptores': all_desc,
        'selected': selected_desc, 'name': atributo.nombre,
        'atributo_id': atributo_id, 'panel_id': panel_id, 'anclas': anclas
        }


@app.route("/paneles/exists/<name>", methods=("GET","POST"))
@logged([admin_permission, lider_permission])
def panel_exists(name):
    panel = Panel.view_all[name]
    if panel:
        return json.dumps(True)
    else:
        return json.dumps(False)

@app.route("/paneles/create", methods=("GET","POST"))
@app.route("/paneles/clone/<panel_id>", methods=("GET","POST"))
@logged([admin_permission, lider_permission])
@templated("upload.html")
def upload_file(panel_id=None):
    ''' First steap of panel creation, upload and parse a file '''
    form = PanelForm()

    ret = {'form': form}

    keys = ('session_id', 'repeticion', 'evaluador', 'muestra', 'rotulo',
                                                        'vasito', 'orden')

    form.lider.choices = [(user.id, user.username)\
                               for user in User.view_role['lider']]

    panel = Panel.view_all[form.nombre.data]
    if panel:
        flash('El nombre de panel ya existe')
        return ret

    if form.validate_on_submit():
        filename = secure_filename(form.orden.file.filename)
        if filename and allowed_file(filename):
            file_path = os.path.join(app.config['UPLOAD_FOLDER'], filename)
            form.orden.file.save(file_path)
            try:
                book = xlrd.open_workbook(file_path)
            except XLRDError:
                flash('El archivo es invalido')
                return ret

            if panel_id:
                panel = Panel.load(panel_id).clone()
                panel.activo = False
                panel.archived = False
                panel.finished = False
            else:
                panel = Panel()

            for field in ('nombre', 'lider'):
                setattr(panel, field, getattr(form, field).data)

            panel.store()
            panel_id = panel.id

            num_ses = book.nsheets

            sessions = []

            for sess_id in xrange(num_ses):
                sheet = book.sheet_by_index(sess_id)
                num_rows, num_cols = sheet.nrows, sheet.ncols
                for rx in xrange(1, num_rows):
                    sess_dict = dict(zip(keys, [i.value \
                                        for i in sheet.row(rx)]))

                    for key, value in sess_dict.iteritems():
                        if key != 'rotulo':
                            sess_dict[key] = int(value)

                    session_panel = Session(panel_id=panel_id, **sess_dict)
                    sessions.append(session_panel)

            # Mass update, speed optimization
            g.couch.update(sessions)
            return redirect('/paneles/create/muestras/%s' % panel_id)
        flash("Se produjo un error")

    return ret



# TODO: Refatorize using FieldList
@app.route("/paneles/create/muestras/<panel_id>", methods=("GET","POST"))
@logged([admin_permission, lider_permission])
@templated("muestras.html")
def crear_muestras(panel_id):
    ''' Create Muestras '''
    panel = Panel.load(panel_id)
    if not panel:
        flash('Ese panel no existe')
        return redirect('/')

    escala_form = EscalaForm()
    if escala_form.validate_on_submit():
        for field in ('min_escala', 'max_escala', 'ancla'):
            value = getattr(escala_form, field).data
            if value != None:
                setattr(panel, field, value)
    else:
        for field in ('min_escala', 'max_escala', 'ancla'):
            value = getattr(panel, field)
            if value != None:
                getattr(escala_form, field).data = value

    # Evaluadores
    users = [(user.id, user.username) for user in User.view_role['user']]
    users += [(user.id, user.username) for user in User.view_role['lider']]

    # Atributos
    attr_disponibles = [attr for attr in Atributo.view_base()]
    attr_by_id = {}
    for attr in attr_disponibles:
        attr_by_id[attr.id] = attr

    attr_asignados = []
    for attr in panel.atributos:
        attr_asignados.append(attr_by_id[attr])

    attr_disponibles = [attr for attr in attr_disponibles \
                                if attr not in attr_asignados]


    # all items ex: [[panel_id, 0]...[panel_id, 99999]]
    start = [panel_id]
    end = [panel_id, {}]

    muestras = [i['key'] for i in \
            Session.view_muestras(group=True)[start:end]]

    formdata = {}

    user_groups = {}

    eval_user = [i['key'] for i in \
            Session.view_evaluador(group=True)[start:end]]

    evaluadores = sorted([int(i[1]) for i in eval_user])
    for user in Session.view_evaluador(group=True)[start:end]:
        if 'group' not in user:
            continue
        user_groups[user['group']] = user

    for user in eval_user:
        formdata['evaluador_%s' % user[1]] = user[2]

    eval_count = len(eval_user)

    fields = [('sel:evaluador_%s' % e, users) for e in evaluadores]

    formdata['tipo'] = panel.tipo
    tipos = [('muestra', 'por muestra'),
             ('atributo', 'por atributo'),
             ('descriptor', 'por descriptor')]
    fields += [('sel:tipo', tipos)]


    form = dynform(fields=fields)(**formdata)

    num_repeticiones = max([doc['key'][1] for doc in \
                        Session.view_rep()[start:end]])

    num_sessions = len(Session.view_panel_ids(reduce=True,
                                    group=True)[start:end])

    rotulos = sorted([doc['key'][1] for doc in \
              Session.view_rotulos()[start:end]],
              cmp=locale.strcoll)
    rotulos = ' ,'.join(rotulos)

    start_m = [panel_id, 1]
    end_m = [panel_id, 1, {}]
    num_max_muestras = len(Session.view_muestras_session()[start_m:end_m])

    ret = {'eval_count': eval_count, 'muestras': muestras,
            'num_max_muestras': num_max_muestras,
            'num_repeticiones': num_repeticiones, 'rotulos': rotulos,
            'num_sesions': num_sessions, 'evaluadores': evaluadores,
            'form': form, 'escala_form':escala_form,
            'attr_disponibles': attr_disponibles,
            'attr_asignados': attr_asignados, 'panel': panel,
            'eval_users': sorted(eval_user, key=lambda item: int(item[1]))}

    sessions = []
    if form.validate_on_submit():
        for session_panel in Session.view_muestras(reduce=False)[start:end]:
            sessions.append(session_panel)
            session_panel.id_evaluador = form['evaluador_%s' % \
                                    session_panel.evaluador].data
        g.couch.update(sessions)
        panel.tipo = form['tipo'].data
        panel.store()

        flash('Se guardo el panel')

        ### TODO: sacar esto
        eval_user = [i['key'] for i in \
                Session.view_evaluador(group=True)[start:end]]

        evaluadores = sorted([int(i[1]) for i in eval_user])

        ret['evaluadores'] = evaluadores
        ret['eval_users'] = sorted(eval_user, key=lambda item: int(item[1]))
        ### /TODO

        return ret

    return ret

@app.route("/session/usuario/close/<panel_id>/<eval_id>/<int:session_id>")
@logged([admin_permission, user_permission, lider_permission])
def close_panel(panel_id, eval_id, session_id):

    key = [eval_id, panel_id, session_id]

    sessions = []
    for sess in Session.view_user_sessions(reduce=False)[key:key]:
        sess.closed = True
        sessions.append(sess)
    g.couch.update(sessions)
    return redirect('/')


@app.route("/paneles/usuario")
@logged([admin_permission, user_permission, lider_permission])
@templated("paneles_usuario.html")
def user_panels():
    user_id = session['userid']
    start = [user_id]
    end = [user_id, {}]

    # User panes
    paneles_id = Session.view_evaluador_panel(
            reduce=True, group=True)[start:end]
    paneles = [Panel.load(p['key'][1]) for p in paneles_id]

    paneles_activos = [panel for panel in paneles if panel and panel.activo]

    paneles_result = {}
    for panel in paneles_activos:
        key = [user_id, panel.id]
        panel_sessions = {}
        for ses in Session.view_evaluador_panel(reduce=False)[key:key].rows:
            panel_sessions[ses['session_id']] = ses

        paneles_result[panel] = panel_sessions
    return {"paneles": paneles_result}


@app.route("/update/atributo/handle_<sess>_<attr>_<desc>/"\
             "<float:value>", methods=("GET", "POST"))
@logged([admin_permission, user_permission, lider_permission])
def update_atributo(sess, attr, desc, value):
    session_panel = Session.load(sess)
    desc = desc.replace("---", "_")

    session_panel.fecha = datetime.datetime.now()

    session_attr = session_panel.atributos.get(attr, {})

    session_attr[desc] = value

    session_panel.atributos[attr] = session_attr
    session_panel.store()
    return ''



@app.route("/session/comment/<panel_id>/<eval_id>/"\
        "<int:session_id>/<int:muestra_id>", methods=("GET", "POST"))
@logged([admin_permission, user_permission, lider_permission])
def session_comment(panel_id, eval_id, session_id, muestra_id):

    key = [eval_id, panel_id, session_id, muestra_id]
    session = Session.view_muestras_session_user(
            include_docs=True, startkey=key, endkey=key)
    session = session.rows[0]

    comment = request.form.get('comment')

    session.comment = comment
    session.store()
    # TODO: better result
    return ''


def sorted_descs(atributo):
    descs = [key for key, val in sorted(atributo.descriptores.iteritems(),
                                   key=lambda i: i[1])]
    return descs

def drange(start, stop, step):
    ''' like xrange but step accept float '''
    current = start
    while current <= stop:
        #oh, h4x
        yield float('%0.1f' % current)
        current += step


@app.route("/session/usuario/<panel_id>/<int:session_id>/<user_id>",
                                                methods=("GET", "POST"))
@app.route("/session/usuario/<panel_id>/<int:session_id>",
                                   methods=("GET", "POST"))
@logged([admin_permission, user_permission, lider_permission])
@templated("user_session.html")
def user_session(panel_id, session_id, user_id=None):
    if not user_id:
        user_id = session['userid']
        preview = False
    else:
        preview = True

    panel = Panel.load(panel_id)

    key = [user_id, panel.id, session_id]

    screens = {}

    vasitos = {}

    sessions = Session.view_user_sessions(reduce=False)[key:key].rows

    for ses in sessions:
        vasitos[ses['vasito']] = ses

    atributos = [Atributo.load(d) for d in  panel.atributos]

    def set_screen(i, atributo, descriptor, sess, panel):
        last_screen = screens.get(i-1, None)
        screen = screens.setdefault(i, {})

        screen['first'] = i == 1 and True or False
        screen['ancla_left'] = atributo.anclas.get("ancla_left_" + descriptor,
                                                                        "Poco")
        screen['ancla_rigth'] = atributo.anclas.get("ancla_rigth_" + descriptor,
                                                                        "Mucho")
        screen['descriptor'] = descriptor
        screen['atributo'] = atributo
        screen['session'] = sess
        screen['last'] = False
        # only usefull in "paneles por muestra"
        screen['end_muestra'] = False

        descriptor = descriptor.replace(' ', '---')

        if atributo.id in sess.atributos and \
                descriptor in sess.atributos[atributo.id]:
            if preview:
                screen['done'] = False
            else:
                screen['done'] = True
        else:
            screen['done'] = False

        if (screen['first'] and screen['done'] == False) or \
                (last_screen and last_screen['done'] == True and \
                                   screen['done'] == False):
            screen['show'] = True
        else:
            screen['show'] = False


        if sess.atributos and atributo.id in sess.atributos and \
                         descriptor in sess.atributos[atributo.id]:
            screen['selected'] = sess.atributos[atributo.id][descriptor]
        elif panel.ancla:
            screen['selected'] = panel.ancla
        else:
            # this should never happend =/
            screen['selected'] = 5
            panel.min_escala = 0
            panel.max_escala = 10
            panel.ancla = 5
            print "but.. happen"

        if preview:
            screen['selected'] = panel.ancla

        screen['selected'] = float(screen['selected'])

        screen['id'] = '%s_%s_%s' % (sess.id, atributo.id, descriptor)

        values = drange(float(panel.min_escala), float(panel.max_escala), 0.1)

        screen['values'] = values

        screen['id'] = screen['id'].replace(' ', '_')
        return screen

    screen = None
    # POR DESCRIPTOR
    if panel.tipo == 'descriptor':
        i = 0
        for atributo in atributos:
            for descriptor in sorted_descs(atributo):
                for sess in sessions:
                    i += 1
                    screen = set_screen(i, atributo, descriptor, sess, panel)
        if screen != None:
            screen['last'] = True


    # POR ATRIBUTO
    elif panel.tipo == 'atributo':
        i = 0
        for atributo in atributos:
            for sess in sessions:
                for descriptor in sorted_descs(atributo):
                    i += 1
                    screen = set_screen(i, atributo, descriptor, sess, panel)
        if screen != None:
            screen['last'] = True

    # POR MUESTRA
    elif panel.tipo == 'muestra':
        i = 0
        for sess in sessions:
            for atributo in atributos:
                for descriptor in sorted_descs(atributo):
                    i += 1
                    screen = set_screen(i, atributo, descriptor, sess, panel)
            screen['end_muestra'] = True
        if screen != None:
            screen['last'] = True
    else:
        raise Exception, 'Invalid panel type'

    context = {"panel": panel, "descriptores": atributos,
        'vasitos': vasitos, 'screens': screens, 'preview': preview, "num_labels": panel.max_escala - panel.min_escala}

    if not screens:
        return redirect('/incomplete')

    return context


@app.route('/incomplete')
@templated("incomplete.html")
def panel_incompleto():
    return {}


@app.route("/login", methods=("GET", "POST"))
@templated("login.html")
def login():
    ''' Login Request '''
    form = LoginForm()

    if form.validate_on_submit():
        user = User.view_all[form.username.data]
        if user:
            user = user.rows[0]
        if user and user.check_password(form.password.data):
            username = user.username
            identity_changed.send(app, identity=Identity(username))
            session['username'] = username
            session['userid'] = user.id
            session['role'] = user.roles[0]
    if 'username' in session:
        return redirect('/')

    return {"form": form}


@app.route("/")
def index():
    if user_permission.can():
        return redirect('/paneles/usuario')
    else:
        return redirect('/paneles')


@app.route("/paneles")
@logged([admin_permission, lider_permission])
@templated("paneles.html")
def paneles():
    ''' Show All Panels '''

    if admin_permission.can():
        paneles = Panel.view_all()
    else:
        username = session['userid']
        paneles = Panel.view_by_lider[username]

    paneles = [panel for panel in paneles]

    closed = {}
    for panel in paneles:
        start_key = [panel.id]
        end_key = [panel.id, []]

        sess = list(Session.view_panel_closed(reduce=True,
                            group=True)[start_key:end_key])
        if sess and not len(sess)>1 and sess[0]['key'][1] == True:
            closed[panel.id] = True
        else:
            closed[panel.id] = False

    return {"paneles": paneles, "closed": closed}


@app.route("/paneles/archived")
@logged([admin_permission, lider_permission])
@templated("paneles.html")
def paneles_archived():
    ''' Show Archived Panels '''
    user_id = session['userid']

    paneles = [panel for panel in Panel.view_archived()]

    closed = {}
    show_panels = []

    is_admin = admin_permission.can()

    for panel in paneles:
        # only admin can view all panels
        if not is_admin:
            if panel.lider != user_id and user_id not in panel.contributors:
                continue
        show_panels.append(panel)

        start_key = [panel.id]
        end_key = [panel.id, []]

        sess = list(Session.view_panel_closed(reduce=True,
                            group=True)[start_key:end_key])
        if not len(sess)>1 and sess[0]['key'][1] == True:
            closed[panel.id] = True
        else:
            closed[panel.id] = False

    return {"is_admin": is_admin, "archive": True, 
             "paneles": show_panels, "closed": closed}


@app.route("/paneles/edit_contributors/<panel_id>", methods=("GET", "POST"))
@logged([admin_permission])
@templated("edit_contributors.html")
def edit_contributors(panel_id):
    panel = Panel.load(panel_id)

    users = [(user.id, user.username) for user in User.view_role['lider']]

    form = ContributorForm()
    form.lider.choices = users

    if form.validate_on_submit():
        panel.contributors = form.lider.data and form.lider.data or []
        panel.store()
    form.lider.data = panel.contributors    

    return {"form": form}

@app.route("/paneles/delete/<panel_id>")
@logged([admin_permission, lider_permission])
@templated("paneles.html")
def delete_panel(panel_id):
    ''' Delete panel using panel_id '''
    panel = Panel.load(panel_id)
    if panel:
        g.couch.delete(panel)
    return redirect('/paneles')

@app.route("/paneles/archive/<panel_id>")
@logged([admin_permission, lider_permission])
@templated("paneles.html")
def archive_panel(panel_id):
    ''' Archive panel using panel_id '''
    panel = Panel.load(panel_id)
    if(panel):
        panel.archived = True
        panel.store()
    return redirect('/paneles')

@app.route("/paneles/activate/<panel_id>", defaults={'active': True})
@app.route("/paneles/deactivate/<panel_id>", defaults={'active': False})
@logged([admin_permission, lider_permission])
def switch_panel(panel_id, active=False):
    ''' Activate/Deactivate panel using panel_id '''
    panel = Panel.load(panel_id)
    if(panel):
        panel.activo = active
        if active == False:
            panel.finished = True
        panel.store()
    return redirect('/paneles')


@app.route("/users")
@logged(admin_permission)
@templated("users.html")
def users():
    ''' List users by role '''
    users_by_role = lambda role: \
            [(user.id, user.username) for user in User.view_role[role]]

    users = {
        'users': users_by_role('user'),
        'admins': users_by_role('admin'),
        'liders': users_by_role('lider'),
        'directors': users_by_role('director')
        }

    return users

@app.route("/users/delete/<user_id>")
@logged(admin_permission)
def delete_user(user_id):
    ''' Delete an user using user_id '''
    user = User.load(user_id)
    if(user):
        g.couch.delete(user)
    return redirect('/users')

def select_default_hack(group, group_field, groups):
    ''' An ugly way to do that '''
    group = (group, group)
    groups.remove(group)
    groups.insert(0, group)
    group_field.choices = groups

@app.route("/users/create", methods=("GET","POST"))
@app.route("/users/edit/<user_id>", methods=("GET","POST"))
@logged(admin_permission)
@templated("create_user.html")
def create_user(user_id=None):
    ''' Create or Edit a user '''
    form = CreateUserForm()

    groups = [(g['key'], g['key']) for g in User.view_group(group=True)]
    groups += [('--NEW--', '-- crear nuevo --')]

    form.group.choices = groups

    if form.validate_on_submit():
        if user_id:
            user = User.load(user_id)
        else:
            user = User()

        if user.group:
            select_default_hack(user.group, form.group, groups)

        if user.username != form.username.data and\
                  User.view_all[form.username.data]:
            flash("El usuario existe")
            return {"form": form, "edit": user_id}

        if '--NEW--' == form.group.data:
            if not form.new_group.data:
                flash("Debe ingresar el nombre del nuevo grupo")
                return {"form": form, "edit": user_id}
            user.group = form.new_group.data
        else:
            user.group = form.group.data

        user.username = form.username.data
        user.roles = [form.role.data]

        if not form.password.data and not user_id:
            flash("Debe ingresar una clave")
            return {"form": form, "edit": user_id}

        elif not user_id or (user_id and form.password.data):
            user.set_password(form.password.data)

        user.store()
        return redirect('/users')

    if user_id:
        user = User.load(user_id)
        if user:
            if user.group:
                select_default_hack(user.group, form.group, groups)

            role = form.role.data
            role = role != 'None' and role or user.roles[0]
            form.username.data = form.username.data or user.username
            form.role.data = role

    return {"form": form, "edit": user_id}


@app.route("/logout")
def logout():
    ''' Logout from system '''
    user = session.pop('username')
    logout = '%s ya no se encuentra logueado en el sistema' % user
    flash(logout)
    return redirect('/')


@app.route("/paneles/get_xls/<panel_id>")
@logged([admin_permission, lider_permission])
def export_xls(panel_id):

    # Create workbook and worksheet
    wbk = xlwt.Workbook()
    sheet = wbk.add_sheet('Tuitos')


    header = ['SESION', 'Panel_Idpanel', 'Nombre', 'Fecha', 'Hora', 'REP',
              'Tope', 'EVALUA', 'IdEvaluador', 'MUESTRA', 'ROTULO',
              'VASITO', 'ORDEN']

    panel = Panel.load(panel_id)
    panel_id = panel.id
    panel_nombre = panel.nombre

    panel_start = [panel_id]
    panel_end = [panel_id, []]

    atributos = [Atributo.load(i) for i in panel.atributos]

    for atributo in atributos:
        for desc in sorted_descs(atributo):
            header.append(atributo.nombre.upper()+desc)
    header.append('Comentario')

    for i, h in enumerate(header):
        sheet.write(0, i, h)

    users = {}
    row = 0
    for i in Session.view_panel_ids(reduce=False)[panel_start:panel_end]:
        row += 1

        user = users.get(i.id_evaluador, User.load(i.id_evaluador))
        sheet.write(row, header.index('Panel_Idpanel'), panel_id)
        sheet.write(row, header.index('Nombre'), panel_nombre)

        fecha = i.fecha.strftime('%d/%m/%Y')
        hora = i.fecha.strftime('%H:%M:%S')

        sheet.write(row, header.index('Fecha'), fecha)
        sheet.write(row, header.index('Hora'), hora)

        sheet.write(row, header.index('SESION'), i.session_id)
        sheet.write(row, header.index('REP'), i.repeticion)
        sheet.write(row, header.index('Tope'), i.tope)
        sheet.write(row, header.index('EVALUA'), i.evaluador)
        sheet.write(row, header.index('IdEvaluador'), user.username)
        sheet.write(row, header.index('MUESTRA'), i.muestra)
        sheet.write(row, header.index('ROTULO'), i.rotulo)
        sheet.write(row, header.index('VASITO'), i.vasito)
        sheet.write(row, header.index('ORDEN'), i.orden)

        for atributo in atributos:
            for desc in sorted_descs(atributo):
                key = atributo.nombre.upper()+desc
                # TODO: must be somewere else
                desc = desc.replace(' ', '_')

                if atributo.id in i.atributos and \
                        desc in i.atributos[atributo.id]:
                    value = float(i.atributos[atributo.id][desc])
                else:
                    value = '*'
                sheet.write(row, header.index(key), value)

        sheet.write(row, header.index('Comentario'), i.comment)

    fd, path = mkstemp()

    wbk.save(path)

    return send_file(path, as_attachment=True, attachment_filename='tuitos.xls', cache_timeout=1)


@app.route("/unautorized")
def unautorized():
    ''' Redirect to / and flash unautorized message '''
    flash('Usted no esta autorizado para acceder aqui!')
    return redirect('/')
